#include <SoftwareSerial.h> 

int pwm_a = 3;                         // Left motor
int dir_a = 12;

int pwm_b = 11;                        // Right motor
int dir_b = 13;

// Channel voice messages (all are 8-bit, 128 and above)
#define   NOTE_OFF               0x80  // 2 bytes: key, velocity
#define   NOTE_ON                0x90  // 2 bytes: key, velocity
#define   CONTROLLER             0xB0  // 2 bytes: controller num, value
#define   CTRL_INSTRUMENT        0xC0  // Set instrument within bank

// Control change messages
#define   CTRL_BANK              0x00  // Select bank of instruments
#define   CTRL_VOLUME            0x07  // Set volune (0-127)
#define   CTRL_ALL_OFF           0x7B  // All notes off

// Instrument banks
#define   BANK_PERCUSSION        0x78  // Percussion (drum) instruments
#define   BANK_MELODIC           0x79  // "Melody" instruments

// General values
#define   Instrument             16    // Instrument (drawbar organ)
#define   NOTE_C4                60    // Middle-C on piano (use with melodic)
#define   NOTE_A4                69    // A above middle-C (concert pitch A)
#define   CHAN_1                 0     // Channel (1 of 16)

// Pin assignments
#define   midiIn                 3     // MIDI input
#define   resetPin               4     // Reset

// Keyboard
byte IRPD[] = {11, 10, 9, 8, A3, A2, A1, A0};
byte IRPDStatus[8];
byte note = NOTE_C4;                   // Root note
byte notes[] = {note, note+2, note+4,  // Eight white key notes
  note+5, note+7, note+9, note+11, note+12};
long previousMillis = 0;
long pauseInterval = 200;              // Min delay for keyboard to clear
byte lastLine = 0;
boolean keyboardClear = true;
boolean fwdRev = false;

// General MIDI settings
byte velocity = 127;                   // Note velocity (volume: 0-127)

SoftwareSerial MIDI(255, midiIn);      // Rx, Tx (Rx not used)

void setup() {
  setupDrive();                        // Set up drive pins  
  Serial.begin(9600);                  // Use to see debug messages
  MIDI.begin(31250);                   // Set up serial comm to MIDI
  resetMidi();                         // Reset MIDI device
  
  ctrlMIDI(CHAN_1, CTRL_VOLUME, velocity);    // Set channel volume
  ctrlMIDI(CHAN_1, CTRL_BANK, BANK_MELODIC);  // Select channel bank
  sendMIDI(CTRL_INSTRUMENT, Instrument, 0);   //Select instrument
}

void loop() {
  byte t = 0;
  for (int i=0; i<8; i++) {                    // Collect all piano keys
    IRPDStatus[i] = !digitalRead(IRPD[i]);
    t <<= 1;                                   // Assemble byte of piano keys
    t |= IRPDStatus[i];
  }
  if(lastLine != t) {                          // Do nothing if no change
    for (int i=0; i<8; i++) {                  // Play piano keys
      if(IRPDStatus[i] == 1)
        noteOn(CHAN_1, notes[i], velocity);    // Note on if key is lit
      else
        noteOff(CHAN_1, notes[i], velocity);   // Note off if key is dark
    }
  }
  lastLine = t;
  action(t);                                   // Control motors by keyboard
  delay(10);
}

void action(byte keyboard) {
  //Serial.println(keyboard);     // Uncomment to see 8-bit value
  
  // Keyboard must be clear for pauseInterval before next command
  if (keyboard == 0) {
    unsigned long currentMillis = millis();
    if(currentMillis - previousMillis > pauseInterval) {
      previousMillis = currentMillis;
      keyboardClear = true;
    }
  } else if (keyboard == 129 || keyboard == 255) {  // Covered
    stopRobot();
    delay(300);
  }
  if (keyboard !=0 && keyboardClear) {  
    keyboardClear = false;
    if( keyboard > 0 && keyboard < 4 )             // LSB 2 bits
      spinLeft();
    else if ( keyboard > 4 && keyboard < 32 ) {    // Center 2 bits
      fwdRev = !fwdRev;                            // Toggle fwd/rev
      if (fwdRev)
        forward();
      else
        reverse();
      delay(250);
    }
    else if ( keyboard > 32 && keyboard <= 128 )   // MSB 2 bits
      spinRight();
  } 
}

// Tuirn note on (press key)
void noteOn(byte channel, byte note, byte attackVelo) {
  sendMIDI( (NOTE_ON | channel), note, attackVelo);
}

// Turn note off (release key)
void noteOff(byte channel, byte note, byte releaseVelo) {
  sendMIDI( (NOTE_OFF | channel), note, releaseVelo);
}

// Set controller value with channel
void ctrlMIDI(byte channel, byte data1, byte data2) {
  sendMIDI( (CONTROLLER | channel), data1, data2);
}

// Send bytes to MIDI device
void sendMIDI(byte cmd, byte data1, byte data2) {
  MIDI.write(cmd);
  MIDI.write(data1);
  // Pass only if valid 2nd byte
  if( (cmd & 0xF0) <= 0xB0 ||  (cmd & 0xF0) == 0xE0 )
    MIDI.write(data2);
}

// Reset MIDI device
void resetMidi() {
  pinMode(resetPin, OUTPUT);
  digitalWrite(resetPin, LOW);
  delay(100);
  digitalWrite(resetPin, HIGH);
  delay(100);
}

// Motion routines for forward, reverse, turns, and stop
void forward() {
  Serial.println("Forward");
  digitalWrite(dir_a, LOW);
  digitalWrite(dir_b, LOW);
  analogWrite(pwm_a, 255); 
  analogWrite(pwm_b, 255);
}
void reverse() {
  Serial.println("Reverse");
  digitalWrite(dir_a, HIGH);
  digitalWrite(dir_b, HIGH);
  analogWrite(pwm_a, 255); 
  analogWrite(pwm_b, 255);
}
void spinLeft() {
  Serial.println("Left");
  digitalWrite(dir_a, HIGH); 
  digitalWrite(dir_b, LOW); 
  setPWM(pwm_a, 70);
  setPWM(pwm_b, 70);
}
void spinRight() {
  Serial.println("Right");
  digitalWrite(dir_a, LOW);
  digitalWrite(dir_b, HIGH);
  setPWM(pwm_a, 70);
  setPWM(pwm_b, 70);
}
void stopRobot() {
  Serial.println("Stop");
  digitalWrite(dir_a, LOW);
  digitalWrite(dir_b, LOW);
  analogWrite(pwm_a, 0); 
  analogWrite(pwm_b, 0);
}

void setupDrive() {
  pinMode(pwm_a, OUTPUT);  //Set control pins to be outputs
  pinMode(pwm_b, OUTPUT);
  pinMode(dir_a, OUTPUT);
  pinMode(dir_b, OUTPUT);
}

void setPWM(int pin, int val) {
  analogWrite(pin, map(val, 0, 100, 0, 255));
}

